""" Temp module until astro units are added to quantities
"""
from quantities import *

L_s = solar_luminosity = UnitQuantity(
    'solar_luminosity',
    3.839*(10**26)*W,
    symbol='L_s',
)
L_s.latex_symbol = 'L_\odot'

R_s = solar_radius = UnitLength(
    'solar_radius',
    6.995 * (10**8) * m,
    aliases=['solar_radii'],
    symbol='R_s',
)
R_s.latex_symbol = 'R_\odot'

R_e = earth_radius = UnitLength(
    'earth_radius',
    6.371 * (10**6) * m,
    aliases=['earth_radii'],
    symbol='R_e',
)
R_e.latex_symbol = 'R_\oplus'

R_j = jupiter_radius = UnitLength(
    'jupiter_radius',
    6.9911 * (10**7) * m,
    aliases=['jupiter_radii'],
    symbol='R_j',
)
R_j.latex_symbol = 'R_J'

M_s = solar_mass = UnitMass(
    'solar_mass', 1.99*(10**30)*kg,
    aliases=['solar_masses'],
    symbol='M_s',
)
M_s.latex_symbol = 'M_\odot'

M_e = earth_mass = UnitMass(
    'earth_mass', 5.97219*(10**24)*kg,
    aliases=['earth_masses'],
    symbol='M_e',
)
M_e.latex_symbol = 'M_\oplus'

M_j = jupiter_mass = UnitMass(
    'jupiter_mass', 1.8986*(10**27)*kg,
    aliases=['jupiter_masses'],
    symbol='M_j',
)
M_j.latex_symbol = 'M_J'

Gyear = giga_year = UnitTime(
    'giga_year', 10**9*year,
    symbol='Gyr',
)

JulianDay = julian_day = JD = UnitTime(
    'julian_day', day,
    symbol='JD',
)
""" Note while quantities doesnt directly support units with an offset in most cases Julian Days are treated like days.
It is useful then to know if your working in Julian Days, MJD, BJD etc"""

ModifiedJulianDate = modified_julian_day = MJD = UnitTime(
    'modified_julian_day', day,
    symbol='MJD',
)

# Compound Units
gcm3 = CompoundUnit('g /cm**3')
gcm3.latex_symbol = 'g/cm^3'

kgm3 = CompoundUnit('kg /m**3')
kgm3.latex_symbol = 'kg/m^3'

ms2 = CompoundUnit('m/s**2')
ms2.latex_symbol = 'm/s^2'


""" This module handles any assumptions the rest of the package uses.

Im not sure how to best implement assumptions that are fully flexible to the user. This could be how the planet type
is defined or other characteristics. Im using a simple 'OK' method here, overwriting assumptions would require loading
this module and defining or changing the variables.

If anyone has a good solution to this issue, please create it in a fork or email me!
"""
import numpy as np

from . import astroquantities as aq

# TODO open an issue about this module for community discussion

planetAssumptions = {
    # Contains all planet assumptions, the key refers to the type of assumption and the format of the value can vary
    # based on the type of assumption

    'massType':
        [
            # Planet types are defined by their Mass, using inf as an absolute upper limit
            # the format of the tuples are (mass upperlimit, name). The current format dosn't allow overlaps and must be
            # in order. If you append a value run .sort() after.
            (10 * aq.M_e, 'Super-Earth'),
            (20 * aq.M_e, 'Neptune'),
            (float('inf'), 'Jupiter')
        ],

    'radiusType':  # TODO interface with rest of module
        [
            (3 * aq.R_e, 'Super-Earth'),
            (6 * aq.R_e, 'Neptune'),
            (float('inf'), 'Jupiter')
        ],

    'tempType':
        [
            (800 * aq.K, 'Cold'),
            (2000 * aq.K, 'Warm'),
            (float('inf'), 'Hot'),
        ],

    'mu':  # depends on masstype so takes the masstype as the key, and mu as the value
        {
            'Super-Earth': 18 * aq.atomic_mass_unit,  # TODO these should be more inherently linked to masstype
            'Neptune': 2 * aq.atomic_mass_unit,
            'Jupiter': 2 * aq.atomic_mass_unit
        },

    'albedo':  # depends on temperature so it takes tempType as the key and the albedo as the value
        {
            'Cold': 0.3,
            'Warm': 0.3,
            'Hot': 0.1,
            'Super-Earth': 0.3,
            'Neptune': 0.1,
            'Jupiter': 0.1,
        },

    'density':
    {
        'Super-Earth': 4 * aq.g / aq.cm**3,
        'Neptune': 1.638 * aq.g / aq.cm**3,
        'Jupiter': 1.326 * aq.g / aq.cm**3
    }
}


def planetMassType(mass):
    """ Returns the planet masstype given the mass and using planetAssumptions['massType']
    """

    if mass is np.nan:
        return None

    for massLimit, massType in planetAssumptions['massType']:

        if mass < massLimit:
            return massType


def planetRadiusType(radius):
    """ Returns the planet radiustype given the mass and using planetAssumptions['radiusType']
    """

    if radius is np.nan:
        return None

    for radiusLimit, radiusType in planetAssumptions['radiusType']:

        if radius < radiusLimit:
            return radiusType


def planetTempType(temperature):
    """ Returns the planet masstype given the temperature and using planetAssumptions['tempType']
    """

    for tempLimit, tempType in planetAssumptions['tempType']:

        if temperature < tempLimit:
            return tempType


def planetType(temperature, mass, radius):
    """ Returns the planet type as 'temperatureType massType'
    """

    if mass is not np.nan:
        sizeType = planetMassType(mass)
    elif radius is not np.nan:
        sizeType = planetRadiusType(radius)
    else:
        return None

    return '{0} {1}'.format(planetTempType(temperature), sizeType)


def planetMu(sizeType):
    return planetAssumptions['mu'][sizeType]


def planetAlbedo(tempType):
    return planetAssumptions['albedo'][tempType]


def planetDensity(radiusType):
     return planetAssumptions['density'][radiusType]


""" Contains the flag class, this is desgined to hold types of information about the target. These can be things
entered by the catalogue when assuming values such as 'Temperature calculated' or personal tags like 'Priority Target'

These are designed to be attached to a planet, system, star or binary class in .flags
"""

allowedFlags = ['Calculated Temperature', 'Estimated Mass', 'Calculated SMA', 'Fake', 'Estimated Distance', 'Calculated Period']
"UBVJIHKLMN"
allowedFlags += ['Estimated magU', 'Estimated magB', 'Estimated magV', 'Estimated magJ', 'Estimated magI',
                 'Estimated magH', 'Estimated magK', 'Estimated magL', 'Estimated magM', 'Estimated magN']


class Flags(object):  # or tags? or lists?

    def __init__(self):

        self.flags = set()

    def addFlag(self, flag):

        if flag in allowedFlags:
            self.flags.add(flag)
        else:
            raise InvalidFlag

    def removeFlag(self, flag):

        self.flags.remove(flag)

    def __repr__(self):

        return 'Flags({0})'.format(str(self.flags)[4:-1])

    def __iter__(self):

        return iter(self.flags)


class InvalidFlag(BaseException):
    pass



from setuptools import setup
import codecs
import os
import re
import sys
import multiprocessing  # stops exit fail on setup.py test

kw = {}
if sys.hexversion >= 0x03000000:
    kw['use_2to3'] = True

here = os.path.abspath(os.path.dirname(__file__))


# Read the version number from a source file.
def find_version(*file_paths):
    # Open in Latin-1 so that we avoid encoding errors.
    # Use codecs.open for Python 2 compatibility
    with codecs.open(os.path.join(here, *file_paths), 'r', 'latin1') as f:
        version_file = f.read()

    # The version line must have the form
    # __version__ = 'ver'
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


# Get the long description from the relevant file
with codecs.open(os.path.join(here, 'readme.md'), encoding='utf-8') as f:
    long_description = f.read()

install_requires = ['numpy', 'quantities', 'nose>=1.0', 'matplotlib>=1.3.1',
                    'requests', 'astropy', 'seaborn']
if sys.hexversion < 0x02070000:
    install_requires.append('unittest2')
    install_requires.append('ordereddict')

if sys.hexversion < 0x02070000:
    test_suite = 'exodata.tests.testsuite'  # otherwise skiptests dont work with 2.6, TODO plugin?
else:
    test_suite = 'nose.collector'


setup(
    name="exodata",
    version=find_version('exodata', '__init__.py'),
    description="Exoplanet catalogue interface",
    long_description=long_description,
    url='https://github.com/ryanvarley/exodata',
    author='Ryan Varley',
    author_email='oecpy@ryanvarley.uk',
    license='GPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Astronomy',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2.6',
    ],

    # What does your project relate to?
    # keywords='sample setuptools development',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages.
    packages=['exodata'],

    # List run-time dependencies here.  These will be installed by pip when your
    # project is installed.
    install_requires = install_requires,

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    include_package_data=False,
    zip_safe=False,
    test_suite=test_suite,

    **kw
)
